
package gus05.entity.gus.file.editor.prop.privatekey.editor1;

import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.FileInputStream;
import java.security.PrivateKey;
import java.util.Properties;

import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.text.JTextComponent;

import gus05.framework.core.Entity;
import gus05.framework.core.Outside;
import gus05.framework.core.Service;
import gus05.framework.features.Give;
import gus05.framework.features.Graphic;


public class PrivateKeyEditor1 implements Entity, Graphic, Give {

    public String getName()				{return "gus.file.editor.prop.privatekey.editor1";}
    public String getCreationDate()		{return "2008.06.12";}
    
    
    public static final String KEY_ERROR = "error";
    
    
    private Service fileToPrivateKey;
    private Service areaHolder3;
    private Service propPanel;
    private Service signedMessageEditor;
    private Service compareWithFile;
    
    private Service rebuildPublicKeyFile;
    private Service generateFileSign;
    
    private JTabbedPane tab;
    private JTextComponent messageEditor;
    
    private JButton button_compareWithFile;
    private JButton button_rebuildPublic;
    private JButton button_generateFileSign;
    
    private File file;
    private Properties prop;
    private String owner;
    private PrivateKey privateKey;
    private String errorMessage;
    
    
    
    
    public PrivateKeyEditor1() throws Exception
    {
    	fileToPrivateKey = Outside.service(this,"gus.security.asymkeys.filetoprivatekey");
    	areaHolder3 = Outside.service(this,"gus.graphic.textcomp.buildgui.panel3");
    	rebuildPublicKeyFile = Outside.service(this,"gus.security.asymkeys.rebuildpublickeyfile");
    	generateFileSign = Outside.service(this,"gus.security.asymkeys.tool.generatefilesign");
    	propPanel = Outside.service(this,"gus.security.asymkeys.privatekey.proppanel1");
    	signedMessageEditor = Outside.service(this,"gus.security.asymkeys.signedmessage.editor1");
    	compareWithFile = Outside.service(this,"gus.security.asymkeys.privatekey.comparewithfile");
    	
        messageEditor = (JTextComponent) signedMessageEditor.gui();
        JComponent comp2 = (JComponent) areaHolder3.transform(messageEditor);
         
        button_compareWithFile = new JButton("compare with file");
        button_compareWithFile.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e)
			{compareWithFile();}
		});
        button_rebuildPublic = new JButton("rebuild publickey file");
        button_rebuildPublic.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e)
			{rebuildPublicKeyFile();}
		});
        button_generateFileSign = new JButton("generate file's signature");
        button_generateFileSign.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e)
			{generateFileSign();}
		});
        
        JPanel p_tools = new JPanel(new GridLayout(0,1));
        p_tools.add(button_compareWithFile);
        p_tools.add(button_rebuildPublic);
        p_tools.add(button_generateFileSign);
        
        tab = new JTabbedPane();
        tab.addTab("properties",propPanel.gui());
        tab.addTab("message editor",comp2);
        tab.addTab("tools",p_tools);
        
    }
    
    
    
    
    
    public JComponent gui()
    {return tab;}
    
    
    
    
    
    
    
    
    public void give(Object obj) throws Exception
    {
        file = (File)obj;
        privateKey = buildPrivateKey();
        
        prop = prop();
        owner = prop.getProperty(SECURITY.PRIVATEKEY_OWNER);
        
        if(errorMessage!=null)
        	prop.setProperty(KEY_ERROR,errorMessage);
        
        propPanel.give(prop);
        signedMessageEditor.give(new Object[]{privateKey,owner});
    }
    
    
    
    
    
    private void rebuildPublicKeyFile()
    {
    	try
    	{rebuildPublicKeyFile.give(file);}
    	catch(Exception e)
    	{Outside.err(this,"rebuildPublicKeyFile()",e);}
    }
    
    
    private void generateFileSign()
    {
    	try
    	{generateFileSign.give(privateKey);}
    	catch(Exception e)
    	{Outside.err(this,"showFileSign()",e);}
    }
    
    
    
    private void compareWithFile()
    {
    	try
    	{
    		boolean r = compareWithFile.filter(prop);
    		if(r) JOptionPane.showMessageDialog(null,"Comparison successfully completed");
    		else JOptionPane.showMessageDialog(null,"Comparison FAILED !!");
    	}
    	catch(Exception e)
    	{Outside.err(this,"compareWithFile()",e);}
    }
    
    
    
    
    

    private PrivateKey buildPrivateKey()
    {
    	errorMessage = null;
    	try{return (PrivateKey) fileToPrivateKey.transform(file);}
    	catch(Exception e)
    	{
    		Outside.err(this,"buildPrivateKey()",e);
    		errorMessage = toString(e);
    		return null;
    	}
    }



    public static String toString(Exception e)
    {
        if(e.toString().startsWith("java.lang.Exception")) return e.getMessage();
        else return e.toString();
    }
    
    
    
    
    private Properties prop() throws Exception
    {
    	Properties prop = new Properties();
    	FileInputStream fis = new FileInputStream(file);
    	prop.load(fis);
    	fis.close();
    	return prop;
    }
}
