
package gus05.entity.gus.file.editor.prop.publickey.editor1;

import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileReader;
import java.security.PublicKey;
import java.util.Properties;

import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.JTextArea;
import javax.swing.text.JTextComponent;

import gus05.framework.core.Entity;
import gus05.framework.core.Outside;
import gus05.framework.core.Service;
import gus05.framework.features.Give;
import gus05.framework.features.Graphic;


public class PublicKeyEditor1 implements Entity, Graphic, Give {

    public String getName()				{return "gus.file.editor.prop.publickey.editor1";}
    public String getCreationDate()		{return "2008.06.11";}
    
    
    public static final String KEY_ERROR = "error";
    
    
    private Service fileToPublicKey;
    private Service textAreaHolder3;
    private Service textAreaHolder3_1;
    private Service propPanel;
    private Service signedMessageChecker;
    private Service compareWithClipboard;
    
    private Service verifyFileSign;
    private Service generateFileMD;
    
    private JTabbedPane tab;
    private JTextArea area_export;
    private JTextComponent area_check;
    
    private JButton button_compareWithClipboard;
    private JButton button_verifyFileSign;
    private JButton button_generateFileMD;
    
    
    private File file;
    private String text;
    private Properties prop;
    private PublicKey publicKey;
    private String owner;
    private String errorMessage;
    
    
    
    
    public PublicKeyEditor1() throws Exception
    {
    	fileToPublicKey = Outside.service(this,"gus.security.asymkeys.filetopublickey");
    	textAreaHolder3 = Outside.service(this,"gus.graphic.textcomp.buildgui.panel3");
    	textAreaHolder3_1 = Outside.service(this,"gus.graphic.textcomp.buildgui.panel3_1");
    	propPanel = Outside.service(this,"gus.security.asymkeys.publickey.proppanel1");
    	signedMessageChecker = Outside.service(this,"gus.security.asymkeys.signedmessage.checker1");
    	compareWithClipboard = Outside.service(this,"gus.security.asymkeys.publickey.comparewithclipboard");
    	
    	verifyFileSign = Outside.service(this,"gus.security.asymkeys.tool.verifyfilesign");
    	generateFileMD = Outside.service(this,"gus.security.messagedigest.tool.generatefilemd");
    	
        area_export = new JTextArea();
        area_export.setEditable(false);
        area_export.setLineWrap(true);
        
        area_check = (JTextComponent) signedMessageChecker.gui();
        
        JComponent comp1 = (JComponent) textAreaHolder3.transform(area_export);
        JComponent comp2 = (JComponent) textAreaHolder3_1.transform(area_check);
        
        button_compareWithClipboard = new JButton("compare with clipboard");
        button_compareWithClipboard.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e)
			{compareWithClipboard();}
		});
        button_verifyFileSign = new JButton("verify file's signature");
        button_verifyFileSign.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e)
			{verifyFileSign();}
		});
        button_generateFileMD = new JButton("generate file's message digest");
        button_generateFileMD.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e)
			{generateFileMD();}
		});
        JPanel p_tools = new JPanel(new GridLayout(0,1));
        p_tools.add(button_compareWithClipboard);
        p_tools.add(button_verifyFileSign);
        p_tools.add(button_generateFileMD);
        
        tab = new JTabbedPane();
        tab.addTab("properties",propPanel.gui());
        tab.addTab("check message",comp2);
        tab.addTab("export key",comp1);
        tab.addTab("tools",p_tools);
    }
    
    
    
    
    
    
    
    
    
    public JComponent gui()
    {return tab;}
    
    
    
    
    
    
    
    
    public void give(Object obj) throws Exception
    {
        file = (File)obj;
        publicKey = buildPublicKey();
        
        text = text();
        prop = prop();
        owner = prop.getProperty(SECURITY.PUBLICKEY_OWNER);
        
        if(errorMessage!=null)
        	prop.setProperty(KEY_ERROR,errorMessage);
        
        propPanel.give(prop);
        signedMessageChecker.give(new Object[]{publicKey,owner});
        
        area_export.setText(text);
        area_export.moveCaretPosition(0);
    }
    
    
    
    
    
    
    private PublicKey buildPublicKey()
    {
    	errorMessage = null;
    	try{return (PublicKey) fileToPublicKey.transform(file);}
    	catch(Exception e)
    	{
    		Outside.err(this,"buildPublicKey()",e);
    		errorMessage = toString(e);
    		return null;
    	}
    }

    
    
    public static String toString(Exception e)
    {
        if(e.toString().startsWith("java.lang.Exception")) return e.getMessage();
        else return e.toString();
    }
    
    
    
    
    
    
    
    
    
    
    
    
    private String text() throws Exception
    {
        if(file==null)return "";
        FileReader fr = new FileReader(file);	
        char[] a= new char[(int)file.length()];
        fr.read(a,0,(int)file.length());
        fr.close();
        return new String(a); 
    }
    
    
    private Properties prop() throws Exception
    {
    	Properties prop = new Properties();
    	FileInputStream fis = new FileInputStream(file);
    	prop.load(fis);
    	fis.close();
    	return prop;
    }
    
    
    
    
    
    
    
    private void verifyFileSign()
    {
    	try
    	{
    		verifyFileSign.give(publicKey);
    	}
    	catch(Exception e)
    	{Outside.err(this,"verifyFileSign()",e);}
    }
    
    
    
    
    
    
    private void generateFileMD()
    {
    	try
    	{
    		generateFileMD.execute();
    	}
    	catch(Exception e)
    	{Outside.err(this,"generateFileMD()",e);}
    }
    
    
    
    
    
    private void compareWithClipboard()
    {
    	try
    	{
    		boolean r = compareWithClipboard.filter(prop);
    		if(r) JOptionPane.showMessageDialog(null,"Comparison successfully completed");
    		else JOptionPane.showMessageDialog(null,"Comparison FAILED !!");
    	}
    	catch(Exception e)
    	{Outside.err(this,"compareWithClipboard()",e);}
    }
}
