package gus05.entity.gus.security.asymkeys.generatekeys;



import java.security.KeyFactory;
import java.security.KeyPair;
import java.security.KeyPairGenerator;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.security.Signature;
import java.security.spec.DSAPrivateKeySpec;
import java.security.spec.DSAPublicKeySpec;
import java.util.Properties;

import gus05.framework.core.Entity;
import gus05.framework.core.Outside;
import gus05.framework.core.Service;
import gus05.framework.features.Transform;



public class GenerateKeys implements Entity, Transform {

    
    public String getName() {return "gus.security.asymkeys.generatekeys";}
    public String getCreationDate() {return "2008.06.10";}
    
    
    
    private Service base64Encoder;
    
    private Signature si;
    private KeyFactory kf;
    private KeyPairGenerator kpg;
    
    
    
    
    public GenerateKeys() throws Exception
    {
    	base64Encoder = Outside.service(this,"gus.tostring.bytetobase64");
    	
    	si = Signature.getInstance(SECURITY.SECURITY_ALGO_DSA);
    	kf = KeyFactory.getInstance(SECURITY.SECURITY_ALGO_DSA);
    	kpg = KeyPairGenerator.getInstance(SECURITY.SECURITY_ALGO_DSA);
        kpg.initialize(512);
    }

    
    
    
    
    public Object transform(Object obj) throws Exception
	{
    	String owner = (String) obj;
    	
    	KeyPair kp = kpg.generateKeyPair();
    	
    	PrivateKey privateKey = kp.getPrivate();
    	PublicKey publicKey = kp.getPublic();
    	
    	String[] xpqg = XPQG(privateKey);
    	String[] ypqg = YPQG(publicKey);
    	
    	si.initSign(privateKey);
    	String sign = sign(owner);
    	
    	Properties prop_private = prop_privateKey(xpqg,ypqg,owner,sign);
    	Properties prop_public = prop_publicKey(ypqg,owner,sign);
    	
    	return new Properties[]{prop_public,prop_private};
	}
    
    
    
    private String[] XPQG(PrivateKey privateKey) throws Exception
    {
    	DSAPrivateKeySpec spec = (DSAPrivateKeySpec) kf.getKeySpec(privateKey, DSAPrivateKeySpec.class);
    	String[] s = new String[4];
        s[0] = spec.getX().toString();
        s[1] = spec.getP().toString();
        s[2] = spec.getQ().toString();
        s[3] = spec.getG().toString();
        return s;
    }
    
    
    
    private String[] YPQG(PublicKey publicKey) throws Exception
    {
    	DSAPublicKeySpec spec = (DSAPublicKeySpec) kf.getKeySpec(publicKey, DSAPublicKeySpec.class);
        String[] s = new String[4];
        s[0] = spec.getY().toString();
        s[1] = spec.getP().toString();
        s[2] = spec.getQ().toString();
        s[3] = spec.getG().toString();
        return s;
    }
    
    
    
    
    
    
    
    private Properties prop_privateKey(String[] xpqg, String[] ypqg, String owner, String sign) throws Exception
    {
    	Properties prop = new Properties();
        prop.setProperty(SECURITY.PRIVATEKEY_X1, xpqg[0]);
        prop.setProperty(SECURITY.PRIVATEKEY_P1, xpqg[1]);
        prop.setProperty(SECURITY.PRIVATEKEY_Q1, xpqg[2]);
        prop.setProperty(SECURITY.PRIVATEKEY_G1, xpqg[3]);
        
        prop.setProperty(SECURITY.PRIVATEKEY_Y0, ypqg[0]);
        prop.setProperty(SECURITY.PRIVATEKEY_P0, ypqg[1]);
        prop.setProperty(SECURITY.PRIVATEKEY_Q0, ypqg[2]);
        prop.setProperty(SECURITY.PRIVATEKEY_G0, ypqg[3]);
        
        prop.setProperty(SECURITY.PRIVATEKEY_OWNER, owner);
        prop.setProperty(SECURITY.PRIVATEKEY_SIGN, sign);
        
        return prop;
    }
    
    
    
    
    
    private Properties prop_publicKey(String[] ypqg, String owner, String sign) throws Exception
    {
        Properties prop = new Properties();
        prop.setProperty(SECURITY.PUBLICKEY_Y0, ypqg[0]);
        prop.setProperty(SECURITY.PUBLICKEY_P0, ypqg[1]);
        prop.setProperty(SECURITY.PUBLICKEY_Q0, ypqg[2]);
        prop.setProperty(SECURITY.PUBLICKEY_G0, ypqg[3]);
        
        prop.setProperty(SECURITY.PUBLICKEY_OWNER, owner);
        prop.setProperty(SECURITY.PUBLICKEY_SIGN, sign);
        
        return prop;
    }
    
    
    
    
    private String sign(String input) throws Exception
    {
    	si.update(input.getBytes());
        byte[] signData = si.sign();
        return encode(signData);
    }
    
    
    private String encode(byte[] data) throws Exception
    {return (String) base64Encoder.transform(data);} 

} 
