package gus05.entity.gus.security.asymkeys.privatekey.signer;


import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.security.PrivateKey;
import java.security.Signature;

import gus05.framework.core.Entity;
import gus05.framework.core.Outside;
import gus05.framework.core.Service;
import gus05.framework.features.Transform;



public class PrivateKeySigner implements Entity, Transform {


    public String getName()				{return "gus.security.asymkeys.privatekey.signer";}
    public String getCreationDate()		{return "2008.07.01";}

    
    public static int BUFFER = 2048;
    

    private Signature si;
    private Service base64Encoder;
    
    
    public PrivateKeySigner() throws Exception
    {
    	base64Encoder = Outside.service(this,"gus.tostring.bytetobase64");
    	si = Signature.getInstance(SECURITY.SECURITY_ALGO_DSA);
    }



    public Object transform(Object obj) throws Exception
    {
    	PrivateKey key = (PrivateKey) obj;
        return new Signer(key);
    }

    
    
    private class Signer implements Transform
    {
    	private PrivateKey key;
    	public Signer(PrivateKey key) {this.key = key;}
    	
		public Object transform(Object obj) throws Exception
		{
			if(obj instanceof byte[])
	            return sign(key,(byte[])obj);
	        if(obj instanceof String)
	            return sign(key,((String)obj).getBytes());
	        if(obj instanceof InputStream)
	            return sign(key,(InputStream)obj);
	        if(obj instanceof File)
	            return sign(key,new FileInputStream((File)obj));
	        
	        throw new Exception("Invalid input: "+obj.getClass().getName());
		}
    }
    
    
    
    private synchronized String sign(PrivateKey key, byte[] data) throws Exception
    {
    	si.initSign(key);
    	si.update(data);
        byte[] signData = si.sign();
        return encode(signData);
    }
    
    
    private synchronized String sign(PrivateKey key, InputStream is) throws Exception
    {
    	si.initSign(key);
    	int b;
    	byte[] data = new byte[BUFFER];
        while((b = is.read(data,0,BUFFER))!=-1)
        {si.update(data,0,b);}
        byte[] signData = si.sign();
        return encode(signData);
    }

    
    
    private String encode(byte[] data) throws Exception
    {return (String) base64Encoder.transform(data);} 
}

