package gus05.entity.gus.security.asymkeys.proptopublickey;


import java.math.BigInteger;
import java.security.KeyFactory;
import java.security.PublicKey;
import java.security.spec.DSAPublicKeySpec;
import java.security.spec.InvalidKeySpecException;
import java.util.Properties;

import gus05.framework.core.Entity;
import gus05.framework.core.Outside;
import gus05.framework.core.Service;
import gus05.framework.features.Transform;



public class PropToPublicKey implements Entity, Transform {

    
    public String getName() {return "gus.security.asymkeys.proptopublickey";}
    public String getCreationDate() {return "2008.06.13";}
    
    
    
    private Service verifySign;
    private KeyFactory kf;
    
    
    
    public PropToPublicKey() throws Exception
    {
    	verifySign = Outside.service(this,"gus.security.asymkeys.verifysign");
    	kf = KeyFactory.getInstance(SECURITY.SECURITY_ALGO_DSA);
    }

    
    
    public Object transform(Object obj) throws Exception
	{
		Properties prop = (Properties) obj;
		
		if(prop.size()!=SECURITY.PUBLICKEY_INFONUMBER)
			throw new Exception("wrong properties number: "+prop.size());
		
		String y0 = findValue(prop,SECURITY.PUBLICKEY_Y0);
        String p0 = findValue(prop,SECURITY.PUBLICKEY_P0);
        String q0 = findValue(prop,SECURITY.PUBLICKEY_Q0);
        String g0 = findValue(prop,SECURITY.PUBLICKEY_G0);
        String owner = findValue(prop,SECURITY.PUBLICKEY_OWNER);
        String sign = findValue(prop,SECURITY.PUBLICKEY_SIGN);
        
        PublicKey publicKey = buildPublicKey(y0,p0,q0,g0);
        
        verifySign(publicKey,owner,sign);
        return publicKey;
	}
    
    
    
    
    private PublicKey buildPublicKey(String y, String p, String q, String g) throws InvalidKeySpecException
    {
    	DSAPublicKeySpec keySpec = new DSAPublicKeySpec(big(y),big(p),big(q),big(g));
        return kf.generatePublic(keySpec);
    }
    
    
    
    
    private void verifySign(PublicKey publicKey, String owner, String sign) throws Exception
    {verifySign.give(new Object[]{publicKey,owner,sign});}
    
    
    
    
    
    private String findValue(Properties prop, String key) throws Exception
    {
    	if(!prop.containsKey(key))
    		throw new Exception("Invalid public key properties: "+key+" is missing");
    	return prop.getProperty(key);
    }
    

    private BigInteger big(String value)
    {return new BigInteger(value);}
}
